#!/bin/bash

AWK="/usr/bin/awk"
COREADM="/usr/bin/coreadm"
ECHO="/bin/echo"
EGREP="/bin/egrep"
GREP="/usr/bin/grep"
ID="/usr/xpg4/bin/id"
LS="/bin/ls"
NDD="/usr/sbin/ndd"
RM="/bin/rm"
ROUTEADM="/usr/sbin/routeadm"
SED="/usr/bin/sed"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
TOUCH="/bin/touch"
UNAME="/bin/uname"
WC="/bin/wc"

OS="unknown"
OS_VERSION="unknown"

TMP_ROOT="/tmp"
TMP_FILE="${TMP_ROOT}/tmpfile.$$"

SUCCESS=0
ERROR=1

function cleanup {
  ${RM} -f ${TMP_FILE}
}

trap "cleanup; exit ${ERROR}" 1 2 3 15

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function coreadm_configuration {
   print_banner "Checking \"coreadm\" Configuration"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${COREADM} | while read LINE; do
             IDENTIFIER=$( ${ECHO} "${LINE}" | ${SED} 's/^[ ]*\([^:]*\):.*$/\1/' )
             VALUE=$( ${ECHO} "${LINE}" | ${SED} -e 's/^.*:\([^:]*\)$/\1/' -e 's/ //g' )
             case "${IDENTIFIER}" in
                "global core file pattern")
                    if [ "${VALUE}" != "/var/core/core_%n_%f_%u_%g_%t_%p" ]; then
                       print_warn "Incorrect global core file pattern"
                       touch ${TMP_FILE}
                    fi
                    ;;
                "global core file content")
                    if [ "${VALUE}" != "default" ]; then
                       print_warn "Incorrect global core file content"
                       touch ${TMP_FILE}
                    fi
                    ;;
                "init core file pattern")
                    if [ "${VALUE}" != "core" ]; then
                       print_warn "Incorrect init core file pattern"
                       touch ${TMP_FILE}
                    fi
                    ;;
                "init core file content")
                    if [ "${VALUE}" != "default" ]; then
                       print_warn "Incorrect init core file content"
                       touch ${TMP_FILE}
                    fi
                    ;;
                 "global core dumps")
                    if [ "${VALUE}" != "enabled" ]; then
                       print_warn "Global core dumps are disabled"
                       touch ${TMP_FILE}
                    fi
                    ;;
                 "per-process core dumps")
                    if [ "${VALUE}" != "disabled" ]; then
                       print_warn "Per-process core dumps are enabled"
                       touch ${TMP_FILE}
                    fi
                    ;;
                 "global setid core dumps")
                    if [ "${VALUE}" != "enabled" ]; then
                       print_warn "Global SETUID core dumps are disabled"
                       touch ${TMP_FILE}
                    fi
                    ;;
                 "per-process setid core dumps")
                    if [ "${VALUE}" != "disabled" ]; then
                       print_warn "Per-process SETUID core dumps are enabled" 
                       touch ${TMP_FILE}
                    fi
                    ;;
                 "global core dump logging")
                    if [ "${VALUE}" != "enabled" ]; then
                       print_warn "Global core dump logging is disabled"
                       touch ${TMP_FILE}
                    fi
                    ;;
             esac
          done
          if [ -e "${TMP_FILE}" ]; then
             print_warn "One or more issues found whilst parsing _optimum_ coreadm settings"
             print_info "To configure \"recommended\" coreadm settings, issue the following commands:"
             print_info "mkdir -p /var/core"
             print_info "chown root:root /var/core"
             print_info "chmod 700 /var/core"
             print_info "coreadm -g /var/core/core_%n_%f_%u_%g_%t_%p -e log -e global \\"
             print_info "        -e global-setid -d process -d proc-setid" 
          fi
          ${RM} -f ${TMP_FILE}
          ;;
   esac
}

function check_stack_protection {
   print_banner "Checking Stack Protection"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${EGREP} "^set noexec_user_stack[ ]?=[ ]?1$" /etc/system >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "noexec_user_stack not set to 1 in /etc/system"
          else
             print_ok "noexec_user_stack set to 1 in /etc/system"
          fi
          ${EGREP} "^set noexec_user_stack_log[ ]?=[ ]?1$" /etc/system >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "noexec_user_stack_log not set to 1 in /etc/system"
          else
             print_ok "noexec_user_stack_log set to 1 in /etc/system"
          fi
          ;;
   esac 
}

function check_nfs_privileges {
   print_banner "Checking NFS Priviledged Port Restrictions"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${EGREP} "^set nfssrv:nfs_portmon[ ]?=[ ]?1$" /etc/system >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "nfssrv:nfs_portmon not set to 1 in /etc/system"
          else
             print_ok "nfssrv:nfs_portmon is set to 1 in /etc/system"
          fi
          ;;
   esac 
}

function check_tcp_sequence_numbers {
   print_banner "Checking TCP Sequence Numbers"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${GREP} "^TCP_STRONG_ISS=2" /etc/default/inetinit >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "Not using RFC 1948 for TCP initial sequence number generation"
          else
             print_ok "Using RFC 1948 TCP initial sequence number generation"
          fi
          ;;
   esac
}

function check_network_parameters {
   print_banner "Checking Network Parameters"
   print_divider
   NDD_COUNTER=0
   case ${OS_VERSION} in
      "10")
          if [ "$( ${NDD} -get /dev/ip ip_forward_src_routed )" -ne "0" ]; then
             print_warn "ip_forward_src_routed set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_forward_src_routed 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip6_forward_src_routed )" -ne "0" ]; then
             print_warn "ip6_forward_src_routed set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip6_forward_src_routed 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/tcp tcp_rev_src_routes )" -ne "0" ]; then
             print_warn "tcp_rev_src_routes set incorrectly"
             print_info "run: \"ndd -set /dev/tcp tcp_rev_src_routes 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_forward_directed_broadcasts )" -ne "0" ]; then
             print_warn "ip_forward_directed_broadcasts set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_forward_directed_broadcasts 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/tcp tcp_conn_req_max_q0 )" -ne "4096" ]; then
             print_warn "tcp_conn_req_max_q0 set incorrectly"
             print_info "run: \"ndd -set /dev/tcp tcp_conn_req_max_q0 4096\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/tcp tcp_conn_req_max_q )" -ne "1024" ]; then
             print_warn "tcp_conn_req_max_q set incorrectly"
             print_info "run: \"ndd -set /dev/tcp tcp_conn_req_max_q 1024\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_respond_to_timestamp )" -ne "0" ]; then
             print_warn "ip_respond_to_timestamp set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_respond_to_timestamp 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_respond_to_timestamp_broadcast )" -ne "0" ]; then
             print_warn "ip_respond_to_timestamp_broadcast set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_respond_to_timestamp_broadcast 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_respond_to_address_mask_broadcast )" -ne "0" ]; then
             print_warn "ip_respond_to_address_mask_broadcast set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_respond_to_address_mask_broadcast 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_respond_to_echo_broadcast )" -ne "0" ]; then
             print_warn "ip_respond_to_echo_broadcast set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_respond_to_echo_broadcast 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/arp arp_cleanup_interval )" -ne "60000" ]; then
             print_warn "arp_cleanup_interval set incorrectly"
             print_info "run: \"ndd -set /dev/arp arp_cleanup_interval 60000\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_ire_arp_interval )" -ne "60000" ]; then
             print_warn "ip_ire_arp_interval set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_ire_arp_interval 60000\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_ignore_redirect )" -ne "1" ]; then
             print_warn "ip_ignore_redirect set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_ignore_redirect 1\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip6_ignore_redirect )" -ne "1" ]; then
             print_warn "ip6_ignore_redirect set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip6_ignore_redirect 1\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
#          if [ "$( ${NDD} -get /dev/tcp tcp_extra_priv_ports_add )" -ne "6112" ]; then
#             print_warn "tcp_extra_priv_ports_add set incorrectly"
#             print_info "run: \"ndd -set /dev/tcp tcp_extra_priv_ports_add 6112\" on startup"
#             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
#          fi
          if [ "$( ${NDD} -get /dev/ip ip_strict_dst_multihoming )" -ne "1" ]; then
             print_warn "ip_strict_dst_multihoming set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_strict_dst_multihoming 1\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          print_info "The next parameters flagged (if any) should only be modified if this"
          print_info "server will not be performing routing"
          if [ "$( ${NDD} -get /dev/ip ip6_strict_dst_multihoming )" -ne "1" ]; then
             print_warn "ip6_strict_dst_multihoming set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip6_strict_dst_multihoming 1\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip_send_redirects )" -ne "0" ]; then
             print_warn "ip_send_redirects set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip_send_redirects 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          if [ "$( ${NDD} -get /dev/ip ip6_send_redirects )" -ne "0" ]; then
             print_warn "ip6_send_redirects set incorrectly"
             print_info "run: \"ndd -set /dev/ip ip6_send_redirects 0\" on startup"
             (( NDD_COUNTER = NDD_COUNTER + 1 )) 
          fi
          ${ROUTEADM} -p | ${SED} -n '1,4 p' | ${GREP} -i "enabled" >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "IP forwarding is enabled on this server"
             print_info "To disable, execute the following command:"
             print_info "${ROUTEADM} -d ipv4-forwarding -d ipv6-forwarding" 
          else
             print_ok "IP forwarding is disabled"
          fi
          if [ "${NDD_COUNTER}" -ne "0" ]; then
             print_warn "Network Parameters not optimally configured"
             print_info "Create a script to automatically set the correct parameters at boot time"
          else
             print_ok "Network Parameters optimally configured"
          fi
          ;;
   esac 
}

check_os
check_user
print_divider
coreadm_configuration
print_divider
check_stack_protection
print_divider
check_nfs_privileges
print_divider
check_tcp_sequence_numbers
print_divider
check_network_parameters

exit 0

: '
function check_stack_protection {
   print_banner "Checking Stack Protection"
   print_divider
   case ${OS_VERSION} in
      "10")
          ;;
   esac 
}
'
